/*
 * main.c - initializes Impost
 * 
 * Written by ziplock
 * 
 * Comments: sickbeatz@hotmail.com IRC: #b4b0 on EFNet
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "impost.h"
#include "messages.h"
#include "getopt.h"

/* Server port.  */
int             Server_port;
int             Msg_Trigger_Size, Read_Size;

/* Toggles for enabling/disabling different options.  */
impost_boolean  Use_SHCD = FALSE, sniffer = FALSE;
impost_boolean  Disable_hexdump = FALSE;

/* Pathname for Perl script.  */
char           *Script_File;

/* Display help.  */
int
Main_Usage(appname)
	char           *appname;
{
#ifdef HAVE_LIBPCAP
	/* Libpcap's device lookup result.  */
	extern char    *Device;
#endif
	printf("Usage: %s [options ....]\n", appname);
	printf("\nOptions:\n"
	       "  --config=PATH         Configuration file location (%s)\n"
	  "  -s, --script=FILE     Use the perl script `FILE' with Impost\n"
	 "  -p, --port=PORT       Specify a port to use (always required)\n"
	       "  -u, --unusual=SIZE    Trigger a detection when the size of the received\n"
	       "                        message exceeds `SIZE'\n"
	 "  -r, --read-size=SIZE  Maximum read size for incoming messages\n"
	 "  -c, --show-checks     Always show checking messags, even when\n"
	       "                        are smaller than the trigger size\n",
	       DEFAULT_CONFIG_FILE);
#ifdef HAVE_LIBPCAP
	Find_Network_Device();
	printf("\nPacket sniffing\n"
	 "  --sniff               Use packet sniffer instead of operating\n"
	       "                        as a honey pot\n"
	       "  --device=<device>     Device to sniff on (Default: %s)\n", Device);
#endif				/* ifdef HAVE_LIBPCAP */

	printf("\nAdditional options\n"
	       "  --raw-capture         Write the buffer marked suspicious to the capture\n"
	       "                        file `as is' - normally it would be converted into\n"
	       "                        a constant string variable for C\n"
	       "  --disable-hex-table   Do not print hexidecimal tables for messages that\n"
	       "                        contain non-printable characters\n"
	     "  --disable-capture     Disable logging to `capture files'.\n"
	       "  --show-opcodes        Show known operation codes, if any, discovered\n"
	       "                        in a received message\n");

	printf("\nOther options\n"
	   "  -d, --display         Output current configuration settings\n"
	     "  -v, --version         Output version information and exit\n"
	       "  -h, --help            Show this message\n\n");
	printf("Report bugs to <ziplock@x86asm.org>.\n");

	return 0;
}

/*
 * Expand pathnames (ie., convert ~username into /home/username).
 */
char           *
Main_Expand_path(filename)
	char           *filename;
{
	char           *new, *tmp;
	struct passwd  *usr_info;

	if (filename[0] != '~')
		return filename;
	*filename++;

	if (filename[0] != '/')
	{
		if (!strchr(filename, '/'))
			return NULL;

		tmp = (char *)malloc(strlen(filename));
		strcpy(tmp, filename);
		strtok(tmp, "/");

		usr_info = getpwnam(tmp);

		if (!(filename = strchr(filename, '/')))
			return NULL;
	} else
		usr_info = getpwuid(getuid());

	new = (char *)malloc(FILENAME_MAX);
	sprintf(new, "%s%s", usr_info->pw_dir, filename);

	return new;
}

/* Main init function */
int
main(argc, argv)
	int             argc;
	char           *argv[];
{
	int             alt_conf = 0;
	char            c;
	char           *Config_File;
#ifdef HAVE_LIBPCAP
	extern char    *Device;
	extern int      Selected_Device;
#endif
	struct stat     script_stat[1];
	struct stat     conf_stat[1];
	struct stat     capdir_stat[1];

	extern char    *optarg;

	extern impost_boolean Display_conf_setup;
	extern impost_boolean Raw_mode;
	extern impost_boolean Use_Script;
	extern impost_boolean show_matching_opcodes;
	extern impost_boolean disable_capture;
	extern impost_boolean checking_messages;

	extern struct config_options config_opt[];

	Config_File = (char *)malloc(FILENAME_MAX);
	Script_File = (char *)malloc(FILENAME_MAX);
	if (Config_File == 0 || Script_File == 0)
	{
		Impost_PutsF(0, ERROR_MALLOC);
		return 1;
	}
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	Config_File = Main_Expand_path(DEFAULT_CONFIG_FILE);

	do
	{
		static struct option long_options[] =
		{
			{"config", required_argument, 0, 0},
			{"port", required_argument, 0, 'p'},
			{"script", required_argument, 0, 's'},
			{"unusual", required_argument, 0, 'u'},
			{"read-size", required_argument, 0, 'r'},
			{"raw-capture", no_argument, 0, 3},
			{"show-checks", no_argument, 0, 'c'},
			{"display", no_argument, 0, 'd'},
			{"version", no_argument, 0, 'v'},
			{"help", no_argument, 0, 'h'},
			{"disable-hex-table", no_argument, 0, 4},
			{"show-opcodes", no_argument, 0, 5},
			{"disable-capture", no_argument, 0, 6},
#ifdef HAVE_LIBPCAP
			{"sniff", no_argument, 0, 1},
			{"device", required_argument, 0, 2},

#endif				/* ifdef HAVE_LIBPCAP */

			{0, 0, 0, 0}
		};
		int             option_index = 0;
		if ((c = getopt_long(argc, argv, "cvdp:s:r:u:h?", long_options,
				     &option_index)) == -1)
			break;	/* End of options */

		switch (c)
		{
		case 0:	/* Configuration file.  */
			alt_conf = 1;
			Config_File = Main_Expand_path(optarg);
			break;
#ifdef HAVE_LIBPCAP
		case 1:
			sniffer = 1;
			break;
		case 2:
			Selected_Device = 1;
			if (optarg)
				Device = optarg;
			break;
#endif				/* ifdef HAVE_LIBPCAP */

		case 'p':
			Server_port = atoi(optarg);
			break;
		case 'r':
			Read_Size = atoi(optarg);
			break;
		case 'c':
			checking_messages = 1;
			break;
		case 's':
			Script_File = Main_Expand_path(optarg);
			Use_Script = 1;
			break;
		case 'u':
			Msg_Trigger_Size = atoi(optarg);
			if (Msg_Trigger_Size < MTS_SIZE)
			{
				printf(ERROR_MTS_TOO_SMALL);
				return 1;
			}
			break;
		case 3:
			Raw_mode = 1;
			break;
		case 4:
			Disable_hexdump = 1;
			break;
		case 'd':
			Display_conf_setup = 1;
			break;
		case 'v':
			printf("Impost %s\n", VERSION);
			return 0;
		case 5:
			show_matching_opcodes = 1;
			break;
		case 6:
			disable_capture = 1;
			break;
		default:
			Main_Usage(argv[0]);
			return 1;
		}
	}
	while (1);
	printf(IMPOST_BANNER, VERSION, HOST_TYPE);

	if (!Server_port)
	{
		Main_Usage(argv[0]);
		return 1;
	}
	if (SHC_Parse_Defs_File() == -1)
		return 1;

	if (lstat(Config_File, conf_stat) == -1)
	{
		Display_conf_setup = 1;
		printf(ERROR_CONFIG_NOTFOUND, Config_File);
		printf(ERROR_USING_DEFAULTS);
	} else
	{
		if (alt_conf == 1)
			printf(OPTION_CONFIG, Config_File);

		Load_Configuration(Config_File);
	}

	if (lstat(config_opt[C_OPT_CAPTURE_DIRECTORY].option, capdir_stat) == -1)
	{
		printf(ERROR_NOCAPDIR);
#ifdef HAVE_MKDIR
		if (mkdir(config_opt[C_OPT_CAPTURE_DIRECTORY].option,
			  0755) == -1)
		{
			perror("mkdir()");
			return 1;
		}
		printf(CREATED_DIRECTORY,
		       config_opt[C_OPT_CAPTURE_DIRECTORY].option);

#else
		printf(ERROR_NO_MKDIR);
		printf(ERROR_NO_MKDIR2,
		       config_opt[C_OPT_CAPTURE_DIRECTORY].option);
		return 1;
#endif				/* ifdef HAVE_MKDIR */
	}
	if (Msg_Trigger_Size)
		config_opt[C_OPT_TRIGGER_SIZE].value = Msg_Trigger_Size;

	if (Read_Size)
		config_opt[C_OPT_MAX_READ_SIZE].value = Read_Size;

#ifdef HAVE_LIBPCAP
	if (sniffer)
	{
		Impost_Sniff_Loop(Server_port);
		return 0;
	}
#endif				/* ifdef HAVE_LIBPCAP */

	if (strlen(Script_File) > 0)
	{
		if (lstat(Script_File, script_stat) == -1)
			printf(ERROR_SCRIPT_NOTFOUND, Script_File);
		else
		{
			printf(IMPOST_LOAD_SCRIPT, Script_File);

			if (Perl_Load_Script(Script_File))
			{
				fprintf(stderr, ERROR_SCRIPT_FAIL);
				return 1;
			}
		}
	}
	Impost_Create_Server();
	Impost_Server_Loop();

	return 0;
}

/* EOF */
