/*
 * packets.c - Impost's packet sniffer
 * 
 * Written by ziplock
 * 
 * Comments: sickbeatz@hotmail.com IRC: #b4b0 on EFNet
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "impost.h"
#include "messages.h"

/* In case OpenBSD does not set DLT_LOOP to 12.  */
#define	DLT_OBSD_LOOP  12

/* External definitions for handling detections.  */
extern nop_t    Detected_NOP_chr;
extern int      Has_Opcode_Instructions;
extern char     Detected_sled_Type[100];
extern int      SLT;
extern int      x86_sled_pattern_size;

/* Configuration options.  */
extern struct config_options config_opt[];

/* For capture files.  */
struct packet_h Packet_History[5];

/* Find network device.  */
int
Find_Network_Device()
{
	/* let libpcap do all the work */
	Device = pcap_lookupdev(errbuff);
	pcap_lookupnet(Device, &Ipaddr, &Netmask, errbuff);
	return 0;
}

/*
 * Find the datalink type and set the offset for the device we're using.
 */
int
Set_Datalink_Offset(PC_S)
	pcap_t         *PC_S;
{
	switch (pcap_datalink(PC_S))
	{
	case DLT_EN10MB:
	case DLT_EN3MB:
		dl_offset = 14;
		break;
#ifdef DLT_PPP_BSDOS
	case DLT_PPP_BSDOS:
#endif
#ifdef DLT_PPP_SERIAL
	case DLT_PPP_SERIAL:
#endif
	case DLT_PPP:
#ifdef SOLARIS
		dl_offset = 8;
#else				/* !SOLARIS */
		dl_offset = 4;
#endif				/* ifdef SOLARIS */
		break;
#ifdef DLT_SLIP_BSDOS
	case DLT_SLIP_BSDOS:
#endif
	case DLT_SLIP:
#if (FREEBSD || OPENBSD || NETBSD)
		dl_offset = 16;
#else				/* DEFAULT */
		dl_offset = 24;
#endif				/* ifdef FREEBSD || OPENBSD || NETBSD */
		break;
	case DLT_FDDI:
		dl_offset = 21;
		break;
#ifdef DLT_LOOP
	case DLT_LOOP:
#endif
#ifdef OPENBSD
#if DLT_LOOP != 12
	case DLT_OBSD_LOOP:
#endif
#endif				/* ifdef OPENBSD */
	case DLT_NULL:
		dl_offset = 4;
		break;
	case DLT_RAW:
		dl_offset = 0;
		break;
#ifdef DLT_LINUX_SLL
	case DLT_LINUX_SLL:
		dl_offset = 16;
		break;
#endif
	default:		/* Very unlikely, let's hope this isn't the
				 * case */
		Impost_PutsF(0, ".x Unknown data link -- %d! Trying 0 as datalink offset\n",
			     pcap_datalink(PC_S));
		dl_offset = pcap_datalink(PC_S);
		break;
	}
	return 0;
}

/* Clear packet history.  */
int
clear_Packet_History(raddr)
	char           *raddr;
{
	int             i, session_id = 0, found_session = 0;

	for (i = 0; i < 5; i++)
	{
		if (!strcmp(raddr, Packet_History[i].Remoteaddr))
		{
			found_session = TRUE;
			session_id = i;

			break;
		}
	}

	if (!found_session)
		return -1;

	for (i = 0; i < 25; i++)
		*Packet_History[session_id].Data_Segment[i] = FALSE;
	Packet_History[session_id].Count = FALSE;

	return 0;
}

/* Add a data segment to the packet history.  */

int
Add_history(raddr, data_segment, len)
	char           *raddr;
	char           *data_segment;
	int             len;
{
	int             i;
	int             session_id = 0, found_session = 0;

	for (i = 0; i < 5; i++)
	{
		if (!strcmp(raddr, Packet_History[i].Remoteaddr))
		{
			found_session = 1;
			session_id = i;
			break;
		}
	}

	if (!found_session)
	{
		for (i = 0; i < 5; i++)
		{
			if (!Packet_History[i].Count)
			{
				session_id = i;
				strcpy(Packet_History[i].Remoteaddr, raddr);
				break;
			}
			if (i == 5)
				return 0;
		}
	}
	data_segment[len] = '\0';
	len--;

	if (data_segment[len - 1] == '\n' || data_segment[len - 1] == '\r')
		data_segment[len - 1] = '\0';
	else
		data_segment[len] = '\0';

	strncpy(Packet_History[session_id].Data_Segment[Packet_History[session_id].Count],
	       data_segment, PKT_HIST_SIZE);
	Packet_History[session_id].Count++;

	return 0;
}

/*
 * Get session ID for history if it exists. Result will be zero if it
 * doesn't.
 */
int
Get_Session_ID(raddr)
	char           *raddr;
{
	int             i;
	int             session_id = 0, found_session = 0;

	for (i = 0; i < 5; i++)
	{
		if (!strcmp(raddr, Packet_History[i].Remoteaddr))
		{
			found_session = 1;
			session_id = i;
			break;	/* Got it.  */
		}
	}

	return session_id;
}

/* Extract headers and payload from a packet.  */
int
Decode_Packet()
{
	u_short         ip_len;
	u_short         tcp_len;

	ip_hdr = (struct siphdr *) (packet + dl_offset);
	ip_len = (ip_hdr->ihl << 2);
	if (ip_hdr->protocol != 6)
		return -1;	/* Wrong protocol.  */

	tcphdr = (struct stcphdr *) (packet + dl_offset + ip_len);
	tcp_len = (tcphdr->th_off << 2);

	payload = (char *)(packet + dl_offset + ip_len + tcp_len);

	return 0;
}

/* Main loop -- Impost's packet sniffer.  */
int
Impost_Sniff_Loop(dest_port)
	int             dest_port;
{
	extern int      Display_conf_setup;
	int             noprint = 0;
	int             data_seg_len, i, hex;
	nop_t           x86_sled_len = 0, x86_sled_t = 0;

	if (Display_conf_setup)
		Display_Variables();
	if (Selected_Device != 1)
		Find_Network_Device();

	printf(IMPOST_INIT_SNIFFER,
	       Impost_Get_TimeStamp());
	Session = pcap_open_live(Device, 9999, 1, 1000, errbuff);
	if (Session == NULL)
	{
		Impost_PutsF(0, "\n.x pcap_open_live() -- %s\n", errbuff);
		exit(1);
	}
	Set_Datalink_Offset(Session);
	Impost_PutsF(0, ".> Device: %s\n"
		     ".> Destination Port: %d\n", Device, dest_port);

	buffer = (char *)malloc(9999);
	if (buffer == 0)
	{
		Impost_PutsF(0, ERROR_MALLOC);
		exit(1);
	}
	do
	{
		packet = pcap_next(Session, &header);
		if (packet != NULL)
		{
			if (Decode_Packet() == -1)
				continue;

			if (ntohs(tcphdr->th_dport) == dest_port)
			{
				saddr.s_addr = ip_hdr->saddr;
				switch (tcphdr->th_flags)
				{
				case TH_FIN | TH_ACK:
					Impost_PutsF(0,
						     ".. Packet with flags TH_FIN|TH_ACK from %s (Bye)\n",
						     inet_ntoa(saddr));
					clear_Packet_History(inet_ntoa(saddr));
					break;
				case TH_ACK:
					Impost_PutsF(0, ".. Received ACK: %d from %s\n",
						     ntohs(tcphdr->th_ack), inet_ntoa(saddr));
					break;
				}

				if (!(tcphdr->th_flags & TH_PUSH))
					continue;

				Impost_PutsF(0, ".. Packet, with TH_PUSH flag, from %s\n",
					     inet_ntoa(saddr));
#ifdef DEBUG
				printf(".d ip_hdr->protocol..: %d\n", ip_hdr->protocol);
				printf(".d ip_hdr->tot_len...: %d\n", ntohs(ip_hdr->tot_len));

				printf(".d tcphdr->th_flags..: 0x%x\n", tcphdr->th_flags);
				printf(".d  -TH_FIN.........: %s\n"
				       ".d  -TH_SYN.........: %s\n"
				       ".d  -TH_PUSH........: %s\n"
				       ".d  -TH_ACK.........: %s\n",
				   tcphdr->th_flags & TH_FIN ? "YES" : "NO",
				   tcphdr->th_flags & TH_SYN ? "YES" : "NO",
				  tcphdr->th_flags & TH_PUSH ? "YES" : "NO",
				  tcphdr->th_flags & TH_ACK ? "YES" : "NO");
#endif				/* ifdef DEBUG */
				data_seg_len = (ntohs(ip_hdr->tot_len) -
				(ip_hdr->ihl << 2) - (tcphdr->th_off << 2));

				Impost_PutsF(0, ".i SEQ = %d ACK = %d\n",
					     ntohs(tcphdr->th_seq), ntohs(tcphdr->th_ack));
				Impost_PutsF(0, ".i Payload -- %d bytes\n", strlen(payload));
				Impost_PutsF(0, ".i Payload's data segment is %d bytes\n",
					     data_seg_len);

				if ((tcphdr->th_flags & TH_PUSH) && (tcphdr->th_flags & TH_ACK))
				{
					memset(buffer, 0, 9999);
					strncpy(buffer, payload, data_seg_len);

					for (i = 0; i < strlen(buffer) - 1; i++)
					{
						if (!isprint(buffer[i]) &&
						    buffer[i] != '\n' && buffer[i] != '\r')
						{
							noprint = 1;
							break;
						}
						noprint = 0;
					}

					x86_sled_len = Initial_Scan(buffer, strlen(buffer));
					if ((data_seg_len > config_opt[0].value) ||
					    (Has_Opcode_Instructions))
					{
						Detected_Buffer(buffer, data_seg_len,
								x86_sled_len,
							0, 0, x86_sled_len);

						printf(SP_SAVING);
						Impost_Capture(buffer, Get_Session_ID(inet_ntoa(saddr)),
							       PACKET_CAPTURE, inet_ntoa(saddr));
					} else
					{
						Add_history(inet_ntoa(saddr), buffer,
							    strlen(buffer));
					}

					if (!noprint)
						Impost_PrintIO(IO_TYPE_PACKET, 0,
						    strlen(buffer), buffer);

					else
						Impost_Hexdump(buffer, 0);
				}
			}
		}
	}
	while (1);

	/* pointless */
	return 0;
}

/* EOF */
