/*
 * script.c - perl embedding
 * 
 * Written by ziplock
 * 
 * Comments: sickbeatz@hotmail.com IRC: #b4b0 on EFNet
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "impost.h"
#include "messages.h"

/* Perl headers */
#include <EXTERN.h>
#include <perl.h>

/*
 * In Perl versions prior to version 5.6.0 the functions used by Impost
 * (get_av, call_pv and get_sv) were defined only as perl_get_av,
 * perl_call_pv and perl_get_sv
 */
#ifdef PERL_BEFORE_560

/* For retrieving arrayz */
#ifndef get_av
#define get_av(a,b) perl_get_av(a,b)
#endif
/* For calling sub routines */
#ifndef call_pv
#define call_pv(a, b) perl_call_pv(a, b)
#endif
/* Get a string variable */
#ifndef get_sv
#define get_sv(a, b) perl_get_sv(a, b)
#endif

#endif				/* ifdef PERL_BEFORE_560 */

/* Impost's Perl interpreter */
static PerlInterpreter *my_perl;

/*
 * Load Perl script, should work smoothly unless you suck at Perl
 */
int
Perl_Load_Script(char *Script_Path)
{
	char           *Script_Args[] = {"", Script_Path};

	my_perl = perl_alloc();
	perl_construct(my_perl);
	perl_parse(my_perl, NULL, 2,
		   Script_Args, NULL);
	perl_run(my_perl);
	if (SvTRUE(ERRSV))
		return -1;
	return 0;
}

/*
 * Unload Perl script and free PerlInterpreter
 */
int
Perl_Unload_Script()
{
	perl_destruct(my_perl);
	perl_free(my_perl);
	return 0;
}

/*
 * Call 'on_server_accept' sub routine -- this routine must be defined in any
 * Perl script scripted for Impost
 */
int
Perl_On_Server_Accept(IP_Address, fd)
	char           *IP_Address;
	int             fd;
{
	char           *Buffer = (char *)malloc(256);
	STRLEN          n_a;
	I32             Welcome_Line_count, i;
	AV             *Welcome_message;

	dSP;
	ENTER;

	SAVETMPS;
	PUSHMARK(SP);

	XPUSHs(sv_2mortal(newSVpv(IP_Address, 0)));
	XPUSHs(sv_2mortal(newSViv(fd)));

	PUTBACK;

	call_pv("on_server_accept", G_DISCARD);

	FREETMPS;
	LEAVE;

	Welcome_message = get_av("welcome_message", TRUE);
	Welcome_Line_count = av_len(Welcome_message) + 1;
	for (i = 0; i < Welcome_Line_count; i++)
	{
#ifdef DEBUG
		printf("db:$welcome_message[%d] = '%s'\n",
		       i, SvPV(*av_fetch(Welcome_message,
					 i, FALSE), n_a));
#endif
		strncpy(Buffer, SvPV(*av_fetch(Welcome_message,
					       i, FALSE), n_a), 256);
		Buffer[strlen(Buffer) + 1] = '\n';
		Impost_PrintIO(IO_TYPE_SEND, fd, strlen(Buffer), Buffer);
		send(fd, Buffer, strlen(Buffer), 0);
	}
	av_clear(Welcome_message);
	if (Buffer != NULL)
		free(Buffer);
	Buffer = NULL;
	return 0;
}

/*
 * Call 'on_server_read' sub routine in perl script to decide how to respond
 * to client's input
 */
int 
Perl_On_Server_Read(char *Buffer, int fd)
{
	char           *Buf = (char *)malloc(256);
	STRLEN          n_a;
	AV             *Reply_data;
	SV             *sv_data;
	I32             Reply_Line_count, D_Notice, i;

	dSP;
	ENTER;

	SAVETMPS;
	PUSHMARK(SP);

	XPUSHs(sv_2mortal(newSVpv(Buffer, 0)));
	XPUSHs(sv_2mortal(newSViv(fd)));

	PUTBACK;
	call_pv("on_server_read", G_DISCARD);

	FREETMPS;
	LEAVE;

	Reply_data = get_av("reply_data", TRUE);
	Reply_Line_count = av_len(Reply_data) + 1;
	for (i = 0; i < Reply_Line_count; i++)
	{
		memset(Buf, 0, 256);
		strncpy(Buf, SvPV(*av_fetch(Reply_data,
					    i, FALSE), n_a), 256);
		Impost_PrintIO(IO_TYPE_SEND, fd, strlen(Buf), Buf);
		send(fd, Buf, strlen(Buf), 0);
		memset(Buf, 0, strlen(Buf));
	}

	if (Buf != NULL)
		free(Buf);
	Buf = NULL;
	av_clear(Reply_data);

	sv_data = get_sv("close_connection", TRUE);
	D_Notice = SvIV(sv_data);
	if (D_Notice == 1)
	{
		sv_setiv(sv_data, 0);
		return 11;
	}
	return 0;
}

/* EOF */
