/*
 * conf.c - handles the configuration file
 * 
 * Written by ziplock
 * 
 * Comments: sickbeatz@hotmail.com IRC: #b4b0 on EFNet
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "impost.h"
#include "messages.h"

/* Size limits for the configuration data.  */
#define MAX_LINE_LENGTH      255
#define MAX_VARIABLE_LENGTH  48
#define MAX_VALUE_LENGTH     255

/* Defined in impost.c.  */
extern struct config_options config_opt[];

/* Load configuration file 'path'.  */
int
Load_Configuration(path)
	char           *path;
{
	FILE           *cfg_fd;
	char           *line, *p, *q;
	char           *variable, *value;
	char           *garbage;
	int             line_num, i, x;

	cfg_fd = fopen(path, "r");
	if (cfg_fd == NULL)
	{
		perror("fopen()");
		return -1;
	}
	line = (char *)malloc(MAX_LINE_LENGTH);
	if (line == 0)
	{
		fprintf(stderr, ERROR_MALLOC);
		exit(1);
	}
	memset(line, 0, MAX_LINE_LENGTH);

	variable = (char *)malloc(MAX_VARIABLE_LENGTH);
	value = (char *)malloc(MAX_VALUE_LENGTH);
	garbage = (char *)malloc(MAX_LINE_LENGTH);
	if (variable == 0 || value == 0 || garbage == 0)
	{
		fprintf(stderr, ERROR_MALLOC);
		exit(1);
	}
	line_num = 0;
	do
	{
		line_num++;
		memset(variable, 0, strlen(variable));
		memset(value, 0, strlen(value));

		if (*line == '#' || !*line)
			continue;

		if ((p = strchr(line, '#')))
			*p = 0;
		if ((p = strchr(line, '\n')))
			*p = 0;

		q = line;
		if (line[0] == '\0' || *q == '\0')
			continue;

		if ((p = strchr(line, '=')))
		{
			*p++;
			strcpy(value, p);

			do
			{
				value++;
			}
			while (value[0] == ' ');

			if (value[0] == '\"')
			{
				value++;

				strcpy(garbage, value);
				if (strtok(garbage, "\"") == NULL)
				{
					printf("Config file error.. "
					       "Missing right quotation mark line %d\n",
					       line_num);
					continue;
				}
				strcpy(value, garbage);
			} else
			{
				printf("Config file syntax error line %d\n",
				       line_num);
				continue;
			}
		}
		strcpy(garbage, line);
		strtok(garbage, "=");

		for (i = 0, x = 0; i < strlen(garbage); i++)
		{
			if (garbage[i] != ' ')
			{
				variable[x] = (char)toupper(garbage[i]);
				x++;
			}
		}

		if (variable && value)
		{
			if (!strcmp(variable, "MESSAGE_TRIGGER_SIZE"))
			{
				config_opt[C_OPT_TRIGGER_SIZE].value = atoi(value);
				continue;
			}
			if (!strcmp(variable, "ENABLE_LOGGING"))
			{
				config_opt[C_OPT_ENABLE_LOGGING].value = atoi(value);
				continue;
			}
			if (!strcmp(variable, "LOGFILE"))
			{
				strncpy(config_opt[C_OPT_LOGFILE].option,
					value, strlen(value));
				continue;
			}
			if (!strcmp(variable, "MAX_READ_SIZE"))
			{
				config_opt[C_OPT_MAX_READ_SIZE].value = atoi(value);
				continue;
			}
			if (!strcmp(variable, "CAPTURE_DIRECTORY"))
			{
				strncpy(config_opt[C_OPT_CAPTURE_DIRECTORY].option,
					value, strlen(value));
				continue;
			}
			if (!strcmp(variable, "SHELLCODE_DEFINITIONS"))
			{
				strncpy(config_opt[C_OPT_SHELLCODE_DEFS].option,
					value, strlen(value));
				continue;
			}
			printf("Config Warning: Unknown variable '%s', "
			       "ignoring\n", variable);
		}
	}
	while (fgets(line, MAX_LINE_LENGTH, cfg_fd) != 0);

	free(line);
	fclose(cfg_fd);

	return 0;
}

/* EOF */
