/* impost.h - main header file for Impost
  
   Written by ziplock
   
   Comments: sickbeatz@hotmail.com
   IRC: #b4b0 on EFNet
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */

/* Configuration file created by autoconf.  */
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#ifdef HAVE_UNISTD_H
# include <sys/types.h>
# include <unistd.h>
#endif

#ifdef HAVE_FCNTL
# include <fcntl.h>
#endif

#include <ctype.h>
#include <pwd.h>

#ifdef HAVE_SIGNAL
# include <signal.h>
#endif

#include <time.h>

#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

/* Required for Impost's packet sniffer.  */
#ifdef HAVE_LIBPCAP
# include <pcap.h>
# include "packets.h"
#endif

#ifndef HAVE_MEMSET
# define memset(a, b, c) bzero(a, c)
#endif
#ifndef HAVE_STRCHR 
# define strchr(a, b)  index(a, b)
# define strrchr(a, b) index(a, b)
#endif

/* Default values if configuration file is missing */
#define	DEFAULT_TRIGGER_SIZE		128
#define	DEFAULT_ENABLE_LOGGING		0
#define DEFAULT_LOGFILE			"/tmp/impost.log"
#define DEFAULT_CONFIG_FILE		"~/.impostrc"
#ifndef SHARED_CONF
# define SHARED_CONF			"./impost.conf"
#endif /* ifndef SHARED_CONF */
#define	DEFAULT_CAPTURE_DIRECTORY	"/tmp/impost-captures"
#ifndef DEFAULT_DEFINITIONS
# define DEFAULT_DEFINITIONS		"./impost.shellcodes"
#endif /* ifndef DEFAULT_DEFINITIONS  */

#define DEFAULT_READ_SIZE		9999

/* Capture type -- lets Impost_Capture() know where to 
   read the history from.  */
#define	SERVER_CAPTURE			1
#define	PACKET_CAPTURE			2

/* Boolean values.  */
#define TRUE  1
#define FALSE 0

#define MTS_SIZE 10

typedef int nop_t;
typedef int nop_info;
typedef char x86_nop_data;
typedef long sparc_nop_t;
typedef int impost_boolean;

/* Common operation codes.  */
struct common_opcode {
   char *operation_code; /* The Opcode Instruction */
   char *assembly_code;  /* The assembly code */
   int type;             /* Type of instruction */
   int n_codes;          /* Number of opcodes */
   char *shellcode;
};

/* Configuration variables.  */
struct config_options {
   char variable[128]; /* Variable */
   int  value;         /* Integer value */
   char option[256];   /* String value */
};

/* Packet history.  */
#define PKT_HIST_SIZE 256
struct packet_h
{
   char Remoteaddr[256];       /* Source */
   char Data_Segment[PKT_HIST_SIZE][25]; /* Item */
   int  Count;                 /* Number of items */
};

/* Shellcode definitions.  */
struct shellcodes {
   char ascii_hex_pattern[255];
   char Description[255];
   int Defined; /* To avoid collecting garbage from memory */
};

/* Internationalization.  */
#ifndef NO_GETTEXT
# include <libintl.h>
#else
# include "../intl/libintl.h"
#endif /* HAVE_LIBINTL_H  */

#define _gt(String) gettext(String)
#define gettext_noop(String) String
#define N_(String) gettext_noop(String)

/* X86 Misc.  */
#define X86_OPCODES        51   /* # of defined x86 common codes */

/* NOP sled type pointers */
#define X86_SLED_SINGLE   1  /* \x90\x90\x90\x90 */
#define X86_SLED_PATTERN  2  /* ABCDABCDABCDABCD */
#define X86_SLED_RANDOM   3  /* ABBCHSNZLAIDJFFH */

/* What kind of operation codes were found?  */
#define OC_TYPE_X86     1
#define OC_TYPE_SPARC   2

/* IO_TYPE_* for Impost_PrintIO().  */
#define IO_TYPE_RECV    1
#define IO_TYPE_SEND    2
#define IO_TYPE_PACKET  3

/* List of opcodes which could be mistaken as x86 NOPs.
   Configuration option pointers:  */
#define	C_OPT_TRIGGER_SIZE		0
#define	C_OPT_ENABLE_LOGGING		1
#define	C_OPT_LOGFILE			2
#define	C_OPT_MAX_READ_SIZE		3
#define	C_OPT_CAPTURE_DIRECTORY		4
#define	C_OPT_SHELLCODE_DEFS		5

/* File handling modes  */
#define	F_OPEN_READONLY			"r"
#define F_OPEN_APPEND			"a+"

/* Line column size limit for shellcode
   definitions file.  */
#define SHC_LINE_SIZE			256

/* impost.c -- Impost's main server handlingfunctions.  */
extern int Impost_Create_Server();
extern int Impost_Server_Loop();
extern int Impost_Accept_Connection();
extern int Impost_Receive(int);

/* impost.c -- Buffer handling in different situations.  */
extern int Impost_Capture(char *, int, int, char *);
extern int Impost_Hexdump(char *, int);
extern int Impost_PrintIO(int, int, int, char *);
extern int Impost_PutsF(int, char *, ...);

/* impost.c -- X86 NOP code handling functions */
extern int x86_Verify_NOP(nop_t);
extern int locate_x86_sled(char *);
extern int sled_Type(char *, int);

/* impost.c -- SPARC NOP code checking, etc.  */
extern int Locate_SPARC_nops(char *);

/* impost.c -- Misc */
extern int Initial_Scan(char *, int);
extern int Detected_Buffer(char *, int, int, int, int, int);
extern unsigned long Find_Reversed_x86_Address(char *);
extern char *Impost_Get_TimeStamp();

/* conf.c */
extern int Load_Configuration(char *);

/* script.c -- Load/Unload perl scripts with and for Impost.  */
extern int Perl_Load_Script(char *);
extern int Perl_Unload_Script();

/* For calling and handling sub-routines.  */
extern int Perl_On_Server_Accept(char *, int);
extern int Perl_On_Server_Read(char *, int);

/* shcdefs.c -- Data manipulation:  */
extern char  SHC_Str2H(char *);
extern char *SHC_PutTogetherBuffer(char *);

/* shcdefs.c -- Various detection/definition stuff  */
extern int   SHC_Check_If_Known(char *, int);
extern int   SHC_Parse_Defs_File();
extern int   SHC_Has_X86_Opcode(char *);

/* shextra.h -- Common x86 opcodes found in shellcode  */
extern struct common_opcode x86_common[];

/* base64.c --  Not being used as of yet -- I'll keep it anyways  */
extern char *Base64_Encode(const char *);
extern char *Base64_Decode(const char *);
extern int   Base64_Decode_byte(int);

/* log.c -- Log data  */
extern int write_log(const char *, ...);

/* main.c */
extern char *Main_Expand_path(char *);
extern int Main_Usage(char *);


#ifdef HAVE_LIBPCAP

/* packets.c -- Packet handling  */
extern int Decode_Packet();
extern int Impost_Sniff_Loop(int);

/* packets.c -- Configure datalink type  */
extern int Set_Datalink_Offset(pcap_t *);
extern int Find_Network_Device();

/* packets.c -- Session history   */
extern int clear_Packet_History(char *);
extern int Add_history(char *, char *, int);
extern int Get_Session_ID(char *);

#endif /* ifdef HAVE_LPCAP */

/* EOF */
