/*
 * shcdefs.c - manage shellcode definitions
 * 
 * Written by ziplock
 * 
 * Comments: sickbeatz@hotmail.com IRC: #b4b0 on EFNet
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "impost.h"
#include "messages.h"
#include "shextra.h"

/*
 * When nonzero, display matching opcodes when checking buffers.
 */
int             show_matching_opcodes = 0;

/* Configuration structure.  */
extern struct config_options config_opt[];
/* Shellcode definitions structure.  */
static struct shellcodes Shellcode_Definitions[50];

/* Parse shellcode definition file and allocate defined data.  */
int
SHC_Parse_Defs_File()
{
	char           *Line, *p, *q, *Hex_data, *Desc, *Temp_str;
	int             Line_No = 0;
	int             SHC_Defs_count = 0;

	FILE           *SHC_Defs_File;

	SHC_Defs_File = fopen(config_opt[C_OPT_SHELLCODE_DEFS].option,
			      "r");
	if (SHC_Defs_File == NULL)
	{
		Impost_PutsF(0, SHCDEFS_ERROR_OPEN);

		perror("fopen()");
		return -1;
	}
	Line = (char *)malloc(SHC_LINE_SIZE);
	Hex_data = (char *)malloc(SHC_LINE_SIZE);
	Desc = (char *)malloc(SHC_LINE_SIZE);
	Temp_str = (char *)malloc(SHC_LINE_SIZE);
	if (Line == 0 || Hex_data == 0 || Desc == 0 ||
	    Temp_str == 0)
	{
		fprintf(stderr, ERROR_MALLOC);
		return -1;
	}
	do
	{
		Line_No++;
		memset(Hex_data, 0, strlen(Hex_data));
		memset(Desc, 0, strlen(Desc));

		if (*Line == '#' || !*Line || Line[0] == '#')
			continue;

		if ((p = strchr(Line, '#')))
			*p = 0;

		if ((p = strchr(Line, '\n')))
			*p = 0;

		q = Line;
		if (Line[0] == '\0' || *q == '\0')
			continue;

		if ((p = strchr(Line, ',')))
		{

			*p++;
			strcpy(Desc, p);	/* Description  */
			while (Desc[0] == ' ')
				Desc++;
			if (strtok(Desc, ")") == NULL)
			{
				if (Line_No > 1)
					fprintf(stderr, SHCDEFS_BADSYNTAX, 
						Line_No);
				continue;
			}
		} else
		{
			if (Line_No > 1)
				fprintf(stderr, SHCDEFS_BADSYNTAX,
					Line_No);
			continue;
		}

		strcpy(Temp_str, Line);
		strtok(Temp_str, ",");

		if (Temp_str[0] == '(')
			Temp_str++;

		strcpy(Hex_data, Temp_str);
		if (Hex_data && Desc)
		{
			sprintf(Shellcode_Definitions[SHC_Defs_count].ascii_hex_pattern,
				"%s", Hex_data);
			sprintf(Shellcode_Definitions[SHC_Defs_count].Description,
				"%s", Desc);
			Shellcode_Definitions[SHC_Defs_count].Defined = 1;
			SHC_Defs_count++;
		}
	}
	while (fgets(Line, SHC_LINE_SIZE, SHC_Defs_File) != 0);

	fclose(SHC_Defs_File);
	return 0;
}

/*
 * Return zero for no opcodes; nonzero for opcodes - Several changes
 * contributed by by Graeme Connell.
 */
int
SHC_Has_X86_Opcode(buffer)
	char           *buffer;
{
	int             i, c = 0, b = 0, x, t;
	ssize_t         len, buflen;
	len = strlen(buffer);
	for (x = 0; x < X86_OPCODES; x++)
	{
		/*
		 * don't bother checking substrings towards the end that
		 * would run off the end of the buffer
		 */
		buflen = len - x86_common[x].n_codes;
		for (i = 0; i < buflen; i++)
		{
			/*
			 * check substring starting at position 'i' of buffer
			 * to see if it matches opcode
			 */
			if (!strncmp(buffer + i, x86_common[x].operation_code,
				     x86_common[x].n_codes))
			{
				b += x86_common[x].n_codes;	/* Increase matching
								 * bytes by size of the
								 * current opcode. */

				c++;	/* Increase match count  */
				if (show_matching_opcodes)
					Impost_PutsF(0, ".d Found: (%s) %s -- start position: %d\n",
						     x86_common[x].shellcode,
					    x86_common[x].assembly_code, i);
			}
		}
	}

	return (b > c) ? 1 : 0;
}

/* Convert ASCII hex format to hex buffer.  */
char           *
SHC_PutTogetherBuffer(source)
	char           *source;
{
	char           *Buffer, *Tmp;
	int             Buffer_size = (strlen(source) * 2);
	int             C_hex, i;

	Buffer = (char *)malloc(Buffer_size);
	Tmp = (char *)malloc(2);
	if (Buffer == 0 || Tmp == 0)
	{
		fprintf(stderr, ERROR_MALLOC);
		exit(1);
	}
	memset(Buffer, 0, Buffer_size);

	for (i = 0; i < strlen(source); i++)
	{
		if (source[i] == ' ')
			continue;

		Tmp[0] = source[i];
		Tmp[1] = source[i + 1];
		C_hex = (char)SHC_Str2H(Tmp) & 0xff;
		sprintf(Buffer, "%s%c", Buffer, C_hex);
		i++;
	}

	return Buffer;
}

/*
 * Convert ASCII to Hex... I jacked this function from code of mine I made a
 * few years ago called gH-cgi.c.. I found it on packetstorm a few months
 * ago.
 */
char
SHC_Str2H(Str)
	char           *Str;
{
	register char   ret;

	if (Str[0] >= 'A')
		ret = ((Str[0] & 0xdf) - 'A') + 10;
	else
		ret = (Str[0] - '0');
	ret *= 16;

	if (Str[1] >= 'A')
		ret += ((Str[1] & 0xdf) - 'A') + 10;
	else
		ret += (Str[1] - '0');

	return ret;
}

/*
 * Check if buffer has been mutated by ADM-Mutant Right now only Intel.
 */
int
SHC_CheckADMmutant(TestBuffer, s_len)
	char           *TestBuffer;
	int             s_len;
{
	char           *Code;
	char           *Buffer;

	int             Code_matches = 0;
	int             Junk_len;
	int             Junk_dyn;
	int             Junk_dynint;
	int             Current;
	int             Contains_Common[5];
	int             i;

	Code = (char *)malloc(strlen(TestBuffer));
	if (Code == 0)
	{
		fprintf(stderr, ERROR_MALLOC);
		exit(1);
	}
	Buffer = TestBuffer;

#ifdef DEBUG_MUTATE
	printf("db: ADM Mutate test: checking for Mutate's Morph padding\n");
#endif
	for (i = 0; i < strlen(Buffer); i++)
	{
		for (Current = 0; Current < IA32_JUNKS; Current++)
		{
			Junk_len = intel_njunk[Current].len;
			Junk_dyn = intel_njunk[Current].dyn;
			Junk_dynint = intel_njunk[Current].dynint;

			if (intel_njunk[Current].morph)
			{
				if ((!Junk_dyn && !strncmp(Buffer,
				    intel_njunk[Current].code, Junk_len)) ||
				    (Junk_dyn && !strncmp(Buffer,
				   intel_njunk[Current].code, Junk_dynint)))
				{
					Code_matches++;
					if ((intel_njunk[Current].len - 1) < strlen(Buffer))
						Buffer += intel_njunk[Current].len - 1;
				}
			}
		}
		Buffer++;
	}

#ifdef DEBUG_MUTATE
	printf("db: ADM Mutate test: %d (buffer length / code matches)\n",
	       Code_matches ? strlen(TestBuffer) / Code_matches : 0);
#endif

	if (Code_matches == 0)
		return 0;

	if ((strlen(TestBuffer) / Code_matches) < 4 ||
	    (strlen(TestBuffer) / Code_matches) > 7)
		return 0;

	if (Code_matches > s_len)
		return 0;

	for (i = 0; i < strlen(TestBuffer); i++)
	{
		switch (TestBuffer[i])
		{
		case '[':
			Contains_Common[0] = 1;
			break;
		case ']':
			Contains_Common[1] = 1;
			break;
		case '/':
			Contains_Common[2] = 1;
			break;
		case '?':
			Contains_Common[3] = 1;
			break;
		case '_':
			Contains_Common[4] = 1;
			break;
		default:
			break;
		}
	}

	for (i = 0; i < 4; i++)
		if (Contains_Common[i] != 1)
			return 0;

	return 1;
}

/*
 * Compare the buffer with known shellcode patterns defined in
 * impost.shellcodes
 */
int
SHC_Check_If_Known(Buffer, sled_len)
	char           *Buffer;
	int             sled_len;
{
	char           *Def_Buffer = (char *)malloc(256);
	int             i = 0;
	int             Found = 0;

	if ((i = SHC_CheckADMmutant(Buffer, sled_len)))
	{
		if (i == 1)
			printf(SP_POS_ADMMUTATE);
		return 1;
	}
	while (Shellcode_Definitions[i].Defined == 1)
	{
		Def_Buffer = SHC_PutTogetherBuffer(Shellcode_Definitions[i].ascii_hex_pattern);
		if (strstr(Buffer, Def_Buffer))
		{
			/* word */
			Impost_PutsF(0, SP_SHELLCODE_FOUND,
				     Shellcode_Definitions[i].Description);

			Found = 1;
		}
		i++;
	}

	return Found;
}

/* EOF */
